@@ -2,7 +2,7 @@ import datetime
 import xml.etree.ElementTree as ET
 
 from .shared import SepaPaymentInitn
-from .utils import int_to_decimal_str, make_id
+from .utils import ADDRESS_MAPPING, int_to_decimal_str, make_id
 
 
 class SepaTransfer(SepaPaymentInitn):
@@ -94,6 +94,17 @@ class SepaTransfer(SepaPaymentInitn):
                 del PmtInf_nodes['ReqdExctnDtNode']
 
             PmtInf_nodes['Nm_Dbtr_Node'].text = self._config['name']
+            if payment.get('address', {}):
+                for d, n in ADDRESS_MAPPING:
+                    if self._config['address'].get(d):
+                        n = ET.Element(n)
+                        n.text = self._config['address'][d]
+                        PmtInf_nodes['PstlAdr_Dbtr_Node'].append(n)
+                for line in self._config.get('lines', []):
+                    n = ET.Element('AdrLine')
+                    n.text = line
+                    PmtInf_nodes['PstlAdr_Dbtr_Node'].append(n)
+
             PmtInf_nodes['IBAN_DbtrAcct_Node'].text = self._config['IBAN']
             if 'BIC' in self._config:
                 PmtInf_nodes['BIC_DbtrAgt_Node'].text = self._config['BIC']
@@ -112,6 +123,17 @@ class SepaTransfer(SepaPaymentInitn):
         if bic:
             TX_nodes['BIC_CdtrAgt_Node'].text = payment['BIC']
         TX_nodes['Nm_Cdtr_Node'].text = payment['name']
+        if payment.get('address', {}):
+            for d, n in ADDRESS_MAPPING:
+                if payment['address'].get(d):
+                    n = ET.Element(n)
+                    n.text = payment['address'][d]
+                    TX_nodes['PstlAdr_Cdtr_Node'].append(n)
+            for line in payment['address'].get('lines', []):
+                n = ET.Element('AdrLine')
+                n.text = line
+                TX_nodes['PstlAdr_Cdtr_Node'].append(n)
+
         TX_nodes['IBAN_CdtrAcct_Node'].text = payment['IBAN']
         TX_nodes['UstrdNode'].text = payment['description']
 
@@ -172,6 +194,7 @@ class SepaTransfer(SepaPaymentInitn):
 
         ED['DbtrNode'] = ET.Element("Dbtr")
         ED['Nm_Dbtr_Node'] = ET.Element("Nm")
+        ED['PstlAdr_Dbtr_Node'] = ET.Element("PstlAdr")
         ED['DbtrAcctNode'] = ET.Element("DbtrAcct")
         ED['Id_DbtrAcct_Node'] = ET.Element("Id")
         ED['IBAN_DbtrAcct_Node'] = ET.Element("IBAN")
@@ -195,6 +218,7 @@ class SepaTransfer(SepaPaymentInitn):
         ED['InstdAmtNode'] = ET.Element("InstdAmt")
         ED['CdtrNode'] = ET.Element("Cdtr")
         ED['Nm_Cdtr_Node'] = ET.Element("Nm")
+        ED['PstlAdr_Cdtr_Node'] = ET.Element("PstlAdr")
         ED['CdtrAgtNode'] = ET.Element("CdtrAgt")
         ED['FinInstnId_CdtrAgt_Node'] = ET.Element("FinInstnId")
         if bic:
@@ -225,6 +249,8 @@ class SepaTransfer(SepaPaymentInitn):
             PmtInf_nodes['PmtInfNode'].append(PmtInf_nodes['ReqdExctnDtNode'])
 
         PmtInf_nodes['DbtrNode'].append(PmtInf_nodes['Nm_Dbtr_Node'])
+        if PmtInf_nodes['PstlAdr_Dbtr_Node']:
+            PmtInf_nodes['DbtrNode'].append(TX_nodes['PstlAdr_Dbtr_Node'])
         PmtInf_nodes['PmtInfNode'].append(PmtInf_nodes['DbtrNode'])
 
         PmtInf_nodes['Id_DbtrAcct_Node'].append(PmtInf_nodes['IBAN_DbtrAcct_Node'])
@@ -250,6 +276,8 @@ class SepaTransfer(SepaPaymentInitn):
             TX_nodes['CdtTrfTxInfNode'].append(TX_nodes['CdtrAgtNode'])
 
         TX_nodes['CdtrNode'].append(TX_nodes['Nm_Cdtr_Node'])
+        if TX_nodes['PstlAdr_Cdtr_Node']:
+            TX_nodes['CdbtrNode'].append(TX_nodes['PstlAdr_Cdtr_Node'])
         TX_nodes['CdtTrfTxInfNode'].append(TX_nodes['CdtrNode'])
 
         TX_nodes['Id_CdtrAcct_Node'].append(TX_nodes['IBAN_CdtrAcct_Node'])
@@ -280,6 +308,8 @@ class SepaTransfer(SepaPaymentInitn):
             TX_nodes['CdtTrfTxInfNode'].append(TX_nodes['CdtrAgtNode'])
 
         TX_nodes['CdtrNode'].append(TX_nodes['Nm_Cdtr_Node'])
+        if TX_nodes['PstlAdr_Cdtr_Node']:
+            TX_nodes['CdtrNode'].append(TX_nodes['PstlAdr_Cdtr_Node'])
         TX_nodes['CdtTrfTxInfNode'].append(TX_nodes['CdtrNode'])
 
         TX_nodes['Id_CdtrAcct_Node'].append(TX_nodes['IBAN_CdtrAcct_Node'])
@@ -330,6 +360,16 @@ class SepaTransfer(SepaPaymentInitn):
             else:
                 del PmtInf_nodes['ReqdExctnDtNode']
             PmtInf_nodes['Nm_Dbtr_Node'].text = self._config['name']
+            if self._config.get('address', {}):
+                for d, n in ADDRESS_MAPPING:
+                    if self._config['address'].get(d):
+                        n = ET.Element(n)
+                        n.text = self._config['address'][d]
+                        PmtInf_nodes['PstlAdr_Dbtr_Node'].append(n)
+                for line in self._config['address'].get('lines', []):
+                    n = ET.Element('AdrLine')
+                    n.text = line
+                    PmtInf_nodes['PstlAdr_Dbtr_Node'].append(n)
             PmtInf_nodes['IBAN_DbtrAcct_Node'].text = self._config['IBAN']
 
             if 'BIC' in self._config:
@@ -354,6 +394,8 @@ class SepaTransfer(SepaPaymentInitn):
                 PmtInf_nodes['PmtInfNode'].append(PmtInf_nodes['ReqdExctnDtNode'])
 
             PmtInf_nodes['DbtrNode'].append(PmtInf_nodes['Nm_Dbtr_Node'])
+            if PmtInf_nodes['PstlAdr_Dbtr_Node']:
+                PmtInf_nodes['DbtrNode'].append(PmtInf_nodes['PstlAdr_Dbtr_Node'])
             PmtInf_nodes['PmtInfNode'].append(PmtInf_nodes['DbtrNode'])
 
             PmtInf_nodes['Id_DbtrAcct_Node'].append(PmtInf_nodes['IBAN_DbtrAcct_Node'])
